/*
//
// Expressions for Alasql.js
// Date: 03.11.2014
// (c) 2014, Andrey Gershun
//
*/

/**
  	Expression statement ( = 2*2; )
  	@class
	@param {object} params Initial parameters
*/
yy.ExpressionStatement = function (params) {
	return yy.extend(this, params);
};

/**
	Convert AST to string
	@this ExpressionStatement
	@return {string}
*/
yy.ExpressionStatement.prototype.toString = function () {
	return this.expression.toString();
};
/**
	Execute statement
	@param {string} databaseid Database identificatro
	@param {object} params Statement parameters
	@param {statement-callback} cb Callback
	@return {object} Result value
*/
yy.ExpressionStatement.prototype.execute = function (databaseid, params, cb) {
	if (this.expression) {
		//		console.log(this.expression.toJS('','', null));
		//		console.log(this.expression.toJS('','', null));
		//        console.log(this.expression.toJS('({})','', null));

		alasql.precompile(this, databaseid, params); // Precompile queries
		var exprfn = new Function(
			'params,alasql,p',
			'var y;return ' + this.expression.toJS('({})', '', null)
		).bind(this);
		var res = exprfn(params, alasql);
		if (cb) {
			res = cb(res);
		}
		return res;
	}
};

/**
	Expression class
	@class
	@param {object} params Initial parameters
*/

yy.Expression = function (params) {
	return yy.extend(this, params);
};

/**
	Convert AST to string
	@this ExpressionStatement
	@return {string}
*/
yy.Expression.prototype.toString = function () {
	var s = this.expression.toString();
	if (this.order) {
		s += ' ' + this.order.toString();
	}
	if (this.nocase) {
		s += ' COLLATE NOCASE';
	}
	if (this.direction) {
		s += ' ' + this.direction;
	}
	return s;
};

/**
	Find aggregator in AST subtree
	@this ExpressionStatement
	@param {object} query Query object
*/
yy.Expression.prototype.findAggregator = function (query) {
	if (this.expression.findAggregator) {
		this.expression.findAggregator(query);
	}
};

/**
	Convert AST to JavaScript expression
	@this ExpressionStatement
	@param {string} context Context string, e.g. 'p','g', or 'x'
	@param {string} tableid Default table name
	@param {object} defcols Default columns dictionary
	@return {string} JavaScript expression
*/

yy.Expression.prototype.toJS = function (context, tableid, defcols) {
	//	console.log('Expression',this);
	if (this.expression.reduced) {
		return 'true';
	}
	return this.expression.toJS(context, tableid, defcols);
};

/**
	Compile AST to JavaScript expression
	@this ExpressionStatement
	@param {string} context Context string, e.g. 'p','g', or 'x'
	@param {string} tableid Default table name
	@param {object} defcols Default columns dictionary
	@return {string} JavaScript expression
*/

yy.Expression.prototype.compile = function (context, tableid, defcols) {
	//	console.log('Expression',this);
	if (this.reduced) {
		return returnTrue();
	}
	return new Function('p', 'var y;return ' + this.toJS(context, tableid, defcols));
};

/**
	JavaScript class
	@class
*/
yy.JavaScript = function (params) {
	return yy.extend(this, params);
};
yy.JavaScript.prototype.toString = function () {
	var s = '``' + this.value + '``';
	return s;
};

yy.JavaScript.prototype.toJS = function (/* context, tableid, defcols*/) {
	//	console.log('Expression',this);
	return '(' + this.value + ')';
};
yy.JavaScript.prototype.execute = function (databaseid, params, cb) {
	var res = 1;
	var expr = new Function('params,alasql,p', this.value);
	expr(params, alasql);
	if (cb) {
		res = cb(res);
	}
	return res;
};

/**
	Literal class
	@class
	@example
	MyVar, [My vairable], `MySQL variable`
*/

yy.Literal = function (params) {
	return yy.extend(this, params);
};
yy.Literal.prototype.toString = function () {
	var s = this.value;
	if (this.value1) {
		s = this.value1 + '.' + s;
	}
	//	else s = tableid+'.'+s;
	return s;
};

/**
	Join class
	@class
*/

yy.Join = function (params) {
	return yy.extend(this, params);
};
yy.Join.prototype.toString = function () {
	var s = ' ';
	if (this.joinmode) {
		s += this.joinmode + ' ';
	}
	s += 'JOIN ' + this.table.toString();
	return s;
};

//  yy.Join.prototype.toJS = function(context, tableid) {
//  	return 'JOIN'+this.table.toString();
// }

/**
	Table class
	@class
*/

yy.Table = function (params) {
	return yy.extend(this, params);
};
yy.Table.prototype.toString = function () {
	var s = this.tableid;
	//	if(this.joinmode)
	if (this.databaseid) {
		s = this.databaseid + '.' + s;
	}
	return s;
};

/**
	View class
	@class
*/

yy.View = function (params) {
	return yy.extend(this, params);
};
yy.View.prototype.toString = function () {
	var s = this.viewid;
	//	if(this.joinmode)
	if (this.databaseid) {
		s = this.databaseid + '.' + s;
	}
	return s;
};

/**
	Binary operation class
	@class
*/
yy.Op = function (params) {
	return yy.extend(this, params);
};
yy.Op.prototype.toString = function () {
	if (this.op === 'IN' || this.op === 'NOT IN') {
		return this.left.toString() + ' ' + this.op + ' (' + this.right.toString() + ')';
	}
	if (this.allsome) {
		return (
			this.left.toString() + ' ' + this.op + ' ' + this.allsome + ' (' + this.right.toString() + ')'
		);
	}
	if (this.op === '->' || this.op === '!') {
		var s = this.left.toString() + this.op;
		//		console.log(this.right);

		if (typeof this.right !== 'string' && typeof this.right !== 'number') {
			s += '(';
		}

		s += this.right.toString();

		if (typeof this.right !== 'string' && typeof this.right !== 'number') {
			s += ')';
		}

		return s;
	}
	if (this.op === 'BETWEEN' || this.op === 'NOT BETWEEN') {
		var s =
			this.left.toString() +
			' ' +
			this.op +
			' ' +
			this.right1.toString() +
			' AND ' +
			this.right2.toString();

		return s;
	}
	return (
		this.left.toString() +
		' ' +
		this.op +
		' ' +
		(this.allsome ? this.allsome + ' ' : '') +
		this.right.toString()
	);
};

yy.Op.prototype.findAggregator = function (query) {
	//	console.log(this.toString());
	if (this.left && this.left.findAggregator) {
		this.left.findAggregator(query);
	}
	// Do not go in > ALL
	if (this.right && this.right.findAggregator && !this.allsome) {
		this.right.findAggregator(query);
	}
};

yy.Op.prototype.toType = function (tableid) {
	if (['-', '*', '/', '%', '^'].indexOf(this.op) > -1) {
		return 'number';
	}
	if (['||'].indexOf(this.op) > -1) {
		return 'string';
	}
	if (this.op === '+') {
		if (this.left.toType(tableid) === 'string' || this.right.toType(tableid) === 'string') {
			return 'string';
		}
		if (this.left.toType(tableid) === 'number' || this.right.toType(tableid) === 'number') {
			return 'number';
		}
	}

	if (
		[
			'AND',
			'OR',
			'NOT',
			'=',
			'==',
			'===',
			'!=',
			'!==',
			'!===',
			'>',
			'>=',
			'<',
			'<=',
			'IN',
			'NOT IN',
			'LIKE',
			'NOT LIKE',
			'REGEXP',
			'GLOB',
		].indexOf(this.op) > -1
	) {
		return 'boolean';
	}

	if (
		this.op === 'BETWEEN' ||
		this.op === 'NOT BETWEEN' ||
		this.op === 'IS NULL' ||
		this.op === 'IS NOT NULL'
	) {
		return 'boolean';
	}

	if (this.allsome) {
		return 'boolean';
	}

	if (!this.op) {
		return this.left.toType();
	}

	return 'unknown';
};

yy.Op.prototype.toJS = function (context, tableid, defcols) {
	//	console.log(this);
	var s;
	var refs = [];
	var op = this.op;
	var _this = this;
	//var leftJS = function(){return _this.left.toJS(context,tableid, defcols)};
	//var rightJS = function(){return _this.right.toJS(context,tableid, defcols)};
	var accessedLeft = false,
		accessedRight = false;
	var ref = function (expr) {
		if (expr.toJS) {
			expr = expr.toJS(context, tableid, defcols);
		}
		var i = refs.push(expr) - 1;
		return 'y[' + i + ']';
	};
	var leftJS = function () {
		return ref(_this.left);
	};
	var rightJS = function () {
		return ref(_this.right);
	};

	if (this.op === '=') {
		op = '===';
	} else if (this.op === '<>') {
		op = '!=';
	} else if (this.op === 'OR') {
		op = '||';
	}

	// Arrow operator
	if (this.op === '->') {
		// Expression to prevent error if object is empty (#344)
		var ljs = '(' + leftJS() + '||{})';

		if (typeof this.right === 'string') {
			s = ljs + '["' + this.right + '"]';
		} else if (typeof this.right === 'number') {
			s = ljs + '[' + this.right + ']';
		} else if (this.right instanceof yy.FuncValue) {
			var ss = [];
			if (!(!this.right.args || 0 === this.right.args.length)) {
				var ss = this.right.args.map(ref);
			}
			s = '' + ljs + '[' + JSON.stringify(this.right.funcid) + '](' + ss.join(',') + ')';
		} else {
			s = '' + ljs + '[' + rightJS() + ']';
		}
	}

	if (this.op === '!') {
		if (typeof this.right === 'string') {
			s = '' + 'alasql.databases[alasql.useid].objects[' + leftJS() + ']["' + this.right + '"]';
		}
		// TODO - add other cases
	}

	if (this.op === 'IS') {
		s =
			'' +
			'(' +
			'(' +
			leftJS() +
			'==null)' + // Cant be ===
			' === ' +
			'(' +
			rightJS() +
			'==null)' + // Cant be ===
			')';
	}

	if (this.op === '==') {
		s = '' + 'alasql.utils.deepEqual(' + leftJS() + ',' + rightJS() + ')';
	}

	if (this.op === '===' || this.op === '!===') {
		s =
			'' +
			'(' +
			(this.op === '!===' ? '!' : '') +
			'(' +
			'(' +
			leftJS() +
			').valueOf()' +
			'===' +
			'(' +
			rightJS() +
			').valueOf()' +
			')' +
			')';
	}

	if (this.op === '!==') {
		s = '' + '(!alasql.utils.deepEqual(' + leftJS() + ',' + rightJS() + '))';
	}
	if (this.op === '||') {
		s = '' + "(''+(" + leftJS() + "||'')+(" + rightJS() + '||""))';
	}
	if (this.op === 'LIKE' || this.op === 'NOT LIKE') {
		var s =
			'(' + (this.op === 'NOT LIKE' ? '!' : '') + 'alasql.utils.like(' + rightJS() + ',' + leftJS();
		if (this.escape) {
			s += ',' + ref(this.escape);
		}
		s += '))';
	}
	if (this.op === 'REGEXP') {
		s = 'alasql.stdfn.REGEXP_LIKE(' + leftJS() + ',' + rightJS() + ')';
	}
	if (this.op === 'GLOB') {
		s = 'alasql.utils.glob(' + leftJS() + ',' + rightJS() + ')';
	}

	if (this.op === 'BETWEEN' || this.op === 'NOT BETWEEN') {
		var left = leftJS();
		s =
			'' +
			'(' +
			(this.op === 'NOT BETWEEN' ? '!' : '') +
			'(' +
			'(' +
			ref(this.right1) +
			'<=' +
			left +
			') && (' +
			left +
			'<=' +
			ref(this.right2) +
			')' +
			')' +
			')';

		/*/*
		if(this.right instanceof yy.Op && this.right.op == 'AND') {

			return ref('(('+this.right.left)+'<='+leftJS()+')&&'+
			ref('('+leftJS()+'<='+this.right.right)+'))';

		} else {
			throw new Error('Wrong BETWEEN operator without AND part');
		}
*/
	}

	if (this.op === 'IN') {
		if (this.right instanceof yy.Select) {
			s = '(';
			//			s += 'this.query.queriesdata['+this.queriesidx+']';
			//			s += 'alasql.utils.flatArray(this.query.queriesfn['+(this.queriesidx)+'](params,null,context))';
			s +=
				'alasql.utils.flatArray(this.queriesfn[' +
				this.queriesidx +
				'](params,null,' +
				context +
				'))';
			s += '.indexOf(';
			s += 'alasql.utils.getValueOf(' + leftJS() + '))>-1)';
		} else if (Array.isArray(this.right)) {
			//			if(this.right.length == 0) return 'false';
			s =
				'([' +
				this.right.map(ref).join(',') +
				'].indexOf(alasql.utils.getValueOf(' +
				leftJS() +
				'))>-1)';
			//console.log(s);
		} else {
			s = '(' + rightJS() + '.indexOf(' + leftJS() + ')>-1)';
			//console.log('expression',350,s);
			//		} else {
			//			throw new Error('Wrong IN operator without SELECT part');
		}
	}

	if (this.op === 'NOT IN') {
		if (this.right instanceof yy.Select) {
			s = '(';
			//this.query.queriesdata['+this.queriesidx+']
			//			s += 'alasql.utils.flatArray(this.query.queriesfn['+(this.queriesidx)+'](params,null,p))';
			s += 'alasql.utils.flatArray(this.queriesfn[' + this.queriesidx + '](params,null,p))';
			s += '.indexOf(';
			s += 'alasql.utils.getValueOf(' + leftJS() + '))<0)';
		} else if (Array.isArray(this.right)) {
			//			if(this.right.length == 0) return 'true';
			s = '([' + this.right.map(ref).join(',') + '].indexOf(';
			s += 'alasql.utils.getValueOf(' + leftJS() + '))<0)';
		} else {
			s = '(' + rightJS() + '.indexOf(';
			s += leftJS() + ')==-1)';

			//			throw new Error('Wrong NOT IN operator without SELECT part');
		}
	}

	if (this.allsome === 'ALL') {
		var s;
		if (this.right instanceof yy.Select) {
			//			var s = 'this.query.queriesdata['+this.queriesidx+']';
			s = 'alasql.utils.flatArray(this.query.queriesfn[' + this.queriesidx + '](params,null,p))';

			s += '.every(function(b){return (';
			s += leftJS() + ')' + op + 'b})';
		} else if (Array.isArray(this.right)) {
			s =
				'' +
				(this.right.length == 1 ? ref(this.right[0]) : '[' + this.right.map(ref).join(',') + ']');
			s += '.every(function(b){return (';
			s += leftJS() + ')' + op + 'b})';
		} else {
			throw new Error('NOT IN operator without SELECT');
		}
	}

	if (this.allsome === 'SOME' || this.allsome === 'ANY') {
		var s;
		if (this.right instanceof yy.Select) {
			//			var s = 'this.query.queriesdata['+this.queriesidx+']';
			s = 'alasql.utils.flatArray(this.query.queriesfn[' + this.queriesidx + '](params,null,p))';
			s += '.some(function(b){return (';
			s += leftJS() + ')' + op + 'b})';
		} else if (Array.isArray(this.right)) {
			s =
				'' +
				(this.right.length == 1 ? ref(this.right[0]) : '[' + this.right.map(ref).join(',') + ']');
			s += '.some(function(b){return (';
			s += leftJS() + ')' + op + 'b})';
		} else {
			throw new Error('SOME/ANY operator without SELECT');
		}
	}

	// Special case for AND optimization (if reduced)
	if (this.op === 'AND') {
		if (this.left.reduced) {
			if (this.right.reduced) {
				return 'true';
			} else {
				s = rightJS();
			}
		} else if (this.right.reduced) {
			s = leftJS();
		}

		// Otherwise process as regular operation (see below)
		op = '&&';
	}

	// if(this.op === '^') {
	// 	// return 	'Math.pow('
	// 	// 		+ leftJS()
	// 	// 		+ ','
	// 	// 		+ rightJS()
	// 	// 		+ ')';
	// }

	// Change names
	//	console.log(this);
	var expr = s || '(' + leftJS() + op + rightJS() + ')';

	var declareRefs = 'y=[(' + refs.join('), (') + ')]';

	if (op === '&&' || op === '||' || op === 'IS' || op === 'IS NULL' || op === 'IS NOT NULL') {
		return '(' + declareRefs + ', ' + expr + ')';
	}

	return (
		'(' + declareRefs + ', ' + 'y.some(function(e){return e == null}) ? void 0 : ' + expr + ')'
	);
};

yy.VarValue = function (params) {
	return yy.extend(this, params);
};
yy.VarValue.prototype.toString = function () {
	return '@' + this.variable;
};

yy.VarValue.prototype.toType = function () {
	return 'unknown';
};

yy.VarValue.prototype.toJS = function () {
	return "alasql.vars['" + this.variable + "']";
};

yy.NumValue = function (params) {
	return yy.extend(this, params);
};
yy.NumValue.prototype.toString = function () {
	return this.value.toString();
};

yy.NumValue.prototype.toType = function () {
	return 'number';
};

yy.NumValue.prototype.toJS = function () {
	return '' + this.value;
};

yy.StringValue = function (params) {
	return yy.extend(this, params);
};
yy.StringValue.prototype.toString = function () {
	return "'" + this.value.toString() + "'";
};

yy.StringValue.prototype.toType = function () {
	return 'string';
};

yy.StringValue.prototype.toJS = function () {
	//	console.log("'"+doubleqq(this.value)+"'");
	//	return "'"+doubleqq(this.value)+"'";
	return "'" + escapeq(this.value) + "'";
};

yy.DomainValueValue = function (params) {
	return yy.extend(this, params);
};
yy.DomainValueValue.prototype.toString = function () {
	return 'VALUE';
};

yy.DomainValueValue.prototype.toType = function () {
	return 'object';
};

yy.DomainValueValue.prototype.toJS = function (context, tableid, defcols) {
	//	console.log("'"+doubleqq(this.value)+"'");
	//	return "'"+doubleqq(this.value)+"'";
	return context;
};

yy.ArrayValue = function (params) {
	return yy.extend(this, params);
};
yy.ArrayValue.prototype.toString = function () {
	return 'ARRAY[]';
};

yy.ArrayValue.prototype.toType = function () {
	return 'object';
};

yy.ArrayValue.prototype.toJS = function (context, tableid, defcols) {
	//	console.log("'"+doubleqq(this.value)+"'");
	//	return "'"+doubleqq(this.value)+"'";
	return (
		'[(' +
		this.value
			.map(function (el) {
				return el.toJS(context, tableid, defcols);
			})
			.join('), (') +
		')]'
	);
};

yy.LogicValue = function (params) {
	return yy.extend(this, params);
};
yy.LogicValue.prototype.toString = function () {
	return this.value ? 'TRUE' : 'FALSE';
};

yy.LogicValue.prototype.toType = function () {
	return 'boolean';
};

yy.LogicValue.prototype.toJS = function () {
	return this.value ? 'true' : 'false';
};

yy.NullValue = function (params) {
	return yy.extend(this, params);
};
yy.NullValue.prototype.toString = function () {
	return 'NULL';
};
yy.NullValue.prototype.toJS = function () {
	return 'undefined';
	//	return 'undefined';
};

yy.ParamValue = function (params) {
	return yy.extend(this, params);
};
yy.ParamValue.prototype.toString = function () {
	return '$' + this.param;
};
yy.ParamValue.prototype.toJS = function () {
	if (typeof this.param === 'string') {
		return "params['" + this.param + "']";
	}

	return 'params[' + this.param + ']';
};

yy.UniOp = function (params) {
	return yy.extend(this, params);
};
yy.UniOp.prototype.toString = function () {
	var s;
	s = void 0;
	if (this.op === '~') {
		s = this.op + this.right.toString();
	}
	if (this.op === '-') {
		s = this.op + this.right.toString();
	}
	if (this.op === '+') {
		s = this.op + this.right.toString();
	}
	if (this.op === '#') {
		s = this.op + this.right.toString();
	}
	if (this.op === 'NOT') {
		s = this.op + '(' + this.right.toString() + ')';
	}
	if (this.op === null) {
		s = '(' + this.right.toString() + ')';
	}
	if (!s) {
		s = '(' + this.right.toString() + ')';
	}
	return s;
};

yy.UniOp.prototype.findAggregator = function (query) {
	if (this.right.findAggregator) {
		this.right.findAggregator(query);
	}
};

yy.UniOp.prototype.toType = function () {
	if (this.op === '-') {
		return 'number';
	}

	if (this.op === '+') {
		return 'number';
	}

	if (this.op === 'NOT') {
		return 'boolean';
	}

	// Todo: implement default case
};

yy.UniOp.prototype.toJS = function (context, tableid, defcols) {
	if (this.op === '~') {
		return '(~(' + this.right.toJS(context, tableid, defcols) + '))';
	}

	if (this.op === '-') {
		return '(-(' + this.right.toJS(context, tableid, defcols) + '))';
	}

	if (this.op === '+') {
		return '(' + this.right.toJS(context, tableid, defcols) + ')';
	}

	if (this.op === 'NOT') {
		return '!(' + this.right.toJS(context, tableid, defcols) + ')';
	}

	if (this.op === '#') {
		if (this.right instanceof yy.Column) {
			return "(alasql.databases[alasql.useid].objects['" + this.right.columnid + "'])";
		} else {
			return (
				'(alasql.databases[alasql.useid].objects[' +
				this.right.toJS(context, tableid, defcols) +
				'])'
			);
		}
	}

	// Please avoid === here
	if (this.op == null) {
		// jshint ignore:line
		return '(' + this.right.toJS(context, tableid, defcols) + ')';
	}

	// Todo: implement default case.
};

/*/*
// yy.Star = function (params) { return yy.extend(this, params); }
// yy.Star.prototype.toString = function() {
// 	var s = this.fieldid;
// 	if(this.tableid) {
// 		s = this.tableid+'.'+s;
// 		if(this.databaseid) {
// 			s = this.databaseid+'.'+s;
// 		}
// 	}
// 	if(this.alias) s += ' AS '+this.alias;
// 	return s;
// }
*/
yy.Column = function (params) {
	return yy.extend(this, params);
};
yy.Column.prototype.toString = function () {
	var s;
	if (this.columnid == +this.columnid) {
		// jshint ignore:line
		s = '[' + this.columnid + ']';
	} else {
		s = this.columnid;
	}
	if (this.tableid) {
		if (+this.columnid === this.columnid) {
			s = this.tableid + s;
		} else {
			s = this.tableid + '.' + s;
		}
		if (this.databaseid) {
			s = this.databaseid + '.' + s;
		}
	}
	return s;
};

yy.Column.prototype.toJS = function (context, tableid, defcols) {
	/*/*
//	var s = this.value;
// 	var s = this.columnid;
// 	if(this.tableid) {
// 		s = this.tableid+'.'+s;
// //		if(this.databaseid) {
// //			s = this.databaseid+'.'+s;
// //		}
// 	} else {
// 		s = tableid+'.'+s;
// 	}
*/
	//console.log('yy.Column',this, tableid);
	//	console.log(392,this.columnid);

	//console.log(506,this);

	//console.log(523, this, tableid);

	var s = '';
	if (!this.tableid && tableid === '' && !defcols) {
		if (this.columnid !== '_') {
			s = context + "['" + this.columnid + "']";
		} else {
			if (context === 'g') {
				s = "g['_']";
			} else {
				s = context;
			}
		}
	} else {
		if (context === 'g') {
			// if(this.columnid == '_') {
			// } else {
			s = "g['" + this.nick + "']";
			// }
		} else if (this.tableid) {
			if (this.columnid !== '_') {
				// if() {
				// s = context+'[\''+tableid + '\'][\''+this.tableid+'\'][\''+this.columnid+'\']';
				// } else {
				s = context + "['" + this.tableid + "']['" + this.columnid + "']";
				// }
			} else {
				if (context === 'g') {
					s = "g['_']";
				} else {
					s = context + "['" + this.tableid + "']";
				}
			}
		} else if (defcols) {
			var tbid = defcols[this.columnid];
			if (tbid === '-') {
				throw new Error(
					'Cannot resolve column "' + this.columnid + '" because it exists in two source tables'
				);
			} else if (tbid) {
				if (this.columnid !== '_') {
					s = context + "['" + tbid + "']['" + this.columnid + "']";
				} else {
					s = context + "['" + tbid + "']";
				}
				//			console.log(836,tbid,s);
			} else {
				if (this.columnid !== '_') {
					// if(defcols['.'][this.tableid]) {
					// 	console.log(847,tableid);
					// 	console.log(context+'[\''+tableid + '\'][\''+this.tableid + '\']','[\''+this.columnid+'\']');
					// 	s = context+'[\''+tableid + '\'][\''+this.tableid + '\'][\''+this.columnid+'\']';
					// } else {
					s = context + "['" + (this.tableid || tableid) + "']['" + this.columnid + "']";
					// }
				} else {
					s = context + "['" + (this.tableid || tableid) + "']";
				}
			}
		} else if (tableid === -1) {
			//			if(this.columnid != '') {
			s = context + "['" + this.columnid + "']";
			//			} else {
			//				s = context;
			//			}
		} else {
			if (this.columnid !== '_') {
				s = context + "['" + (this.tableid || tableid) + "']['" + this.columnid + "']";
			} else {
				s = context + "['" + (this.tableid || tableid) + "']";
			}
		}
	}
	//	console.log(context,s);
	//	console.trace(new Error());
	//console.log(874,s);
	return s;
};

yy.AggrValue = function (params) {
	return yy.extend(this, params);
};
yy.AggrValue.prototype.toString = function () {
	var s = '';
	if (this.aggregatorid === 'REDUCE') {
		s += this.funcid.replace(re_invalidFnNameChars, '') + '(';
	} else {
		s += this.aggregatorid + '(';
	}

	if (this.distinct) {
		s += 'DISTINCT ';
	}

	if (this.expression) {
		s += this.expression.toString();
	}

	s += ')';

	if (this.over) {
		s += ' ' + this.over.toString();
	}
	return s;
};

yy.AggrValue.prototype.findAggregator = function (query) {
	//	console.log('aggregator found',this.toString());

	//	var colas = this.as || this.toString();

	var colas = escapeq(this.toString()) + ':' + query.selectGroup.length;
	//	console.log('findAgg',this);

	/*/*	var found = false;
	for(var i=0;i<query.columns.length;i++) {
		// THis part should be intellectual
		if(query.columns[i].as == colas) {
			found = true;
			break;
		}
	}
*/

	//		if(!query.selectColumns[colas]) {
	//		}

	var found = false;

	/*/*
	for(var i=0;i<query.selectGroup.length;i++){
		if(query.selectGroup[i].nick==colas) {
			colas = colas+':'+i;
			found = false;
			break;
		};
	};
*/
	//	console.log("query.selectGroup",query.selectGroup,found);
	if (!found) {
		if (!this.nick) {
			this.nick = colas;
			var found = false;
			for (var i = 0; i < query.removeKeys.length; i++) {
				if (query.removeKeys[i] === colas) {
					found = true;
					break;
				}
			}
			if (!found) {
				query.removeKeys.push(colas);
			}
		}
		query.selectGroup.push(this);
	}
	//	console.log(query.selectGroup);

	//		this.reduced = true;
	return;
};

yy.AggrValue.prototype.toType = function () {
	if (
		['SUM', 'COUNT', 'AVG', 'MIN', 'MAX', 'AGGR', 'VAR', 'STDDEV'].indexOf(this.aggregatorid) > -1
	) {
		return 'number';
	}

	if (['ARRAY'].indexOf(this.aggregatorid) > -1) {
		return 'array';
	}

	if (['FIRST', 'LAST'].indexOf(this.aggregatorid) > -1) {
		return this.expression.toType();
	}

	// todo: implement default;
};

yy.AggrValue.prototype.toJS = function (/*context, tableid, defcols*/) {
	/*/*
//	var s = 'alasql.functions.'+this.funcid+'(';
//	if(this.expression) s += this.expression.toJS(context, tableid);
//	s += ')';
//	if(this.alias) s += ' AS '+this.alias;
//	return s;
//	var s = '';
//if(this.as) console.log(499,this.as);
//	var colas = this.as;
*/
	var colas = this.nick;
	if (colas === undefined) {
		colas = this.toString();
	}
	return "g['" + colas + "']";
};

yy.OrderExpression = function (params) {
	return yy.extend(this, params);
};
yy.OrderExpression.prototype.toString = yy.Expression.prototype.toString;
/*/* //Duplicated code

function() {
	var s = this.expression.toString();
	if(this.order) s += ' '+this.order.toString();
	if(this.nocase) s += ' '+'COLLATE'+' '+'NOCASE';
	return s;
}*/

yy.GroupExpression = function (params) {
	return yy.extend(this, params);
};
yy.GroupExpression.prototype.toString = function () {
	return this.type + '(' + this.group.toString() + ')';
};

/*/* //Duplicated code
yy.ColumnDef = function (params) { return yy.extend(this, params); }
yy.ColumnDef.prototype.toString = function() {
	var s =  this.columnid;
	if(this.dbtypeid) s += ' '+this.dbtypeid;
	if(this.dbsize) {
		s += '('+this.dbsize;
		if(this.dbprecision) s += ','+this.dbprecision;
		s += ')';
	};
	if(this.primarykey) s += ' PRIMARY KEY';
	if(this.notnull) s += ' NOT NULL';
	return s;
}*/
